@@ -257,13 +257,13 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto type = (long) libwps::read16(input);
+	auto type = long(libwps::read16(input));
 	if (type != 0x11)
 	{
 		WPS_DEBUG_MSG(("LotusChart::readChart: not a chart name\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	long endPos=pos+4+sz;
 	f << "Entries(ChartDef):sz=" << sz << ",";
 	if (sz < 0xB2) // find b2 or b3
@@ -274,33 +274,33 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 		ascFile.addNote(f.str().c_str());
 		return true;
 	}
-	auto id=(int) libwps::readU8(input);
+	auto id=int(libwps::readU8(input));
 	auto chart=m_state->getChart(id, *this, stream);
 	f << "id=" << id << ",";
-	librevenge::RVNGString name("");
+	std::string name("");
 	auto fontType=m_mainParser.getDefaultFontType();
 	for (int i=0; i<16; ++i)
 	{
 		unsigned char c = libwps::readU8(input);
 		if (c == '\0') break;
-		libwps::appendUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType), name);
+		name+=char(c);
 	}
 	if (!name.empty())
 	{
-		chart->m_name=name;
-		f << name.cstr() << ",";
+		chart->m_name=libwps_tools_win::Font::unicodeString(name, fontType);
+		f << name << ",";
 	}
 	input->seek(pos+4+17, librevenge::RVNG_SEEK_SET);
 	// group 0: title, 1: axis name, 2: axis data+note+subtitle+legend
 	int val;
 	for (int i=0; i<3; ++i) // useme
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		if (val) f << "font[group" << i << "]=" << val << ",";
 	}
 	for (int i=0; i<6; ++i)
 	{
-		chart->m_serieFormats[i].m_color=val=(int) libwps::readU8(input);
+		chart->m_serieFormats[i].m_color=val=int(libwps::readU8(input));
 		if (!val) continue; // auto
 		if (val==255)
 			f << "serie" << i << "[color]=range,";
@@ -309,11 +309,11 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 		else
 			f << "serie" << i << "[color]=" << val << ",";
 	}
-	val=(int) libwps::read8(input);
+	val=int(libwps::read8(input));
 	if (val) f << "f0=" << val << ",";
 	for (int i=0; i<6; ++i)
 	{
-		chart->m_serieFormats[i].m_hash=val=(int) libwps::readU8(input);
+		chart->m_serieFormats[i].m_hash=val=int(libwps::readU8(input));
 		if (!val) continue;
 		if (val==255)
 			f << "hash[serie" << i << "]=range,";
@@ -328,16 +328,16 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	f << "ChartDef-A:";
 	for (int i=0; i<6; ++i)   // small number
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		if (!val) continue;
 		f << "f" << i << "=" << val << ",";
 	}
 	for (int i=0; i<3; ++i) // useme
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		if (val) f << "fSize[group" << i << "]=" << val << ",";
 	}
-	val=(int) libwps::readU8(input);
+	val=int(libwps::readU8(input));
 	bool showGridY=(val&2);
 	if (val&1) // checkme
 		f << "X,";
@@ -347,7 +347,7 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	chart->getAxis(1).m_showGrid=showGridY;
 	val &= 0xfc;
 	if (val) f << "##grid" <<  "=" << val << ",";
-	val=(int) libwps::readU8(input); // can be 0,1,2
+	val=int(libwps::readU8(input)); // can be 0,1,2
 	if (val&1) f << "color=bw,";
 	val &= 0xfe;
 	if (val) f << "##color" <<  "=" << val << ",";
@@ -383,10 +383,10 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	}
 	f << "type=" << chartType << ",";
 
-	char const *(axisNames[])= {"X","Y","YSecond"};
+	char const *axisNames[]= {"X","Y","YSecond"};
 	for (int i=0; i<3; ++i)
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		// 0=auto
 		if (val==-1)
 		{
@@ -397,14 +397,14 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	}
 	for (auto axisName : axisNames) // useme
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		// 0=auto
 		if (val==-1) f << "setExponent[" << axisName << "]=manual,";
 		else if (val) f << "###setExponent[" << axisName << "]=" << val << ",";
 	}
 	for (auto axisName : axisNames) // useme
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		// 0=auto
 		if (val==-1) f << "legend[" << axisName << "]=manual,";
 		else if (val==1) f << "legend[" << axisName << "]=none,";
@@ -412,7 +412,7 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	}
 	for (int i=0; i<3; ++i)
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		// 0=normal
 		auto &axis=chart->getAxis(i);
 		if (val==1)
@@ -428,7 +428,7 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	}
 	for (auto axisName : axisNames)
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		// 0=auto
 		if (val==-1) f << "setWidth[" << axisName << "]=manual,";
 		else if (val) f << "###setWidth[" << axisName << "]=" << val << ",";
@@ -441,7 +441,7 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	f << "ChartDef-B:";
 	for (int i=0; i<6; ++i)
 	{
-		chart->m_serieFormats[i].m_yAxis=val=(int) libwps::read8(input);
+		chart->m_serieFormats[i].m_yAxis=val=int(libwps::read8(input));
 		if (val==1) continue; // primary axis
 		if (val==2)
 			f << "serie" << i << "[axis]=secondary,";
@@ -450,9 +450,9 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	}
 	for (int i=0; i<6; ++i)   // small number
 	{
-		chart->m_serieFormats[i].m_format=val=(int) libwps::readU8(input);
+		chart->m_serieFormats[i].m_format=val=int(libwps::readU8(input));
 		if (!val) continue;
-		char const *(wh[])= {"both", "lines", "symbols", "neither", "area"};
+		char const *wh[]= {"both", "lines", "symbols", "neither", "area"};
 		if (val<5)
 			f << "serie" << i << "[format]=" << wh[val] << ",";
 		else
@@ -460,9 +460,9 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	}
 	for (int i=0; i<6; ++i)   // small number
 	{
-		chart->m_serieFormats[i].m_align=val=(int) libwps::readU8(input);
+		chart->m_serieFormats[i].m_align=val=int(libwps::readU8(input));
 		if (!val) continue;
-		char const *(wh[])= {"center", "right", "below", "left", "above"};
+		char const *wh[]= {"center", "right", "below", "left", "above"};
 		if (val<5)
 			f << "serie" << i << "[align]=" << wh[val] << ",";
 		else
@@ -476,7 +476,7 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	f << "ChartDef-C:";
 	for (int i=0; i<7; ++i)   // small number expect f18=0|4|64
 	{
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		if (i==0)
 		{
 			chart->getAxis(1).m_showGrid=false;
@@ -556,24 +556,24 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	for (int i=0; i<3; ++i)
 	{
 		// useme
-		val=(int) libwps::read8(input);
+		val=int(libwps::read8(input));
 		if (val) f << "color[group" << i << "]=" << val << ",";
 	}
-	val=(int) libwps::read16(input);
+	val=int(libwps::read16(input));
 	if (val!=1) f << "ticks=" << val << ",";
 	for (auto axisName : axisNames)
 	{
-		val=(int) libwps::read16(input);
+		val=int(libwps::read16(input));
 		if (val!=14) f << "width" << axisName << "=" << val << ",";;
 	}
 	for (int i=0; i<2; ++i)   // small number expect g2=g3=0|1
 	{
-		val=(int) libwps::read16(input);
+		val=int(libwps::read16(input));
 		if (val) f << "g" << i+1 << "=" << val << ",";
 	}
 	for (auto axisName : axisNames)
 	{
-		val=(int) libwps::read16(input);
+		val=int(libwps::read16(input));
 		if (val) f << "exp[manual," << axisName << "]=" << val << ",";;
 	}
 	ascFile.addPos(pos);
@@ -586,11 +586,11 @@ bool LotusChart::readChart(std::shared_ptr<WPSStream> stream)
 	{
 		// useme
 		f << axisName << "=[";
-		val=(int) libwps::readU8(input); // or 2 bytes
+		val=int(libwps::readU8(input)); // or 2 bytes
 		if (val!=0x71) f << "fmt=" << std::hex << val << std::dec << ",";
 		for (int j=0; j<3; ++j)
 		{
-			val=(int) libwps::readU8(input);
+			val=int(libwps::readU8(input));
 			if (val) f << "f" << j << "=" << val << ",";
 		}
 		f << "],";
@@ -638,13 +638,13 @@ bool LotusChart::readChartName(std::shared_ptr<WPSStream> stream)
 	libwps::DebugFile &ascFile=stream->m_ascii;
 	libwps::DebugStream f;
 	long pos = input->tell();
-	auto type = (long) libwps::read16(input);
+	auto type = long(libwps::read16(input));
 	if (type != 0x12)
 	{
 		WPS_DEBUG_MSG(("LotusChart::readChartName: not a chart name\n"));
 		return false;
 	}
-	auto sz = (long) libwps::readU16(input);
+	auto sz = long(libwps::readU16(input));
 	f << "Entries(ChartName):";
 	if (sz < 3)
 	{
@@ -655,22 +655,23 @@ bool LotusChart::readChartName(std::shared_ptr<WPSStream> stream)
 		return true;
 	}
 
-	auto cId=(int) libwps::readU8(input);
+	auto cId=int(libwps::readU8(input));
 	f << "chart[id]=" << cId << ",";
 	auto chart=m_state->getChart(cId, *this, stream);
-	auto id=(int) libwps::readU8(input);
+	auto id=int(libwps::readU8(input));
 	f << "data[id]=" << id << ",";
-	librevenge::RVNGString name("");
+	std::string name("");
 	auto fontType=m_mainParser.getDefaultFontType();
 	for (int i = 0; i < sz-2; i++)
 	{
 		unsigned char c = libwps::readU8(input);
 		if (c == '\0') break;
-		libwps::appendUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType), name);
+		name+=char(c);
 	}
-	f << name.cstr() << ",";
+	f << name << ",";
 	if (!name.empty())
 	{
+		auto uniName=libwps_tools_win::Font::unicodeString(name, fontType);
 		switch (id)
 		{
 		case 0:
@@ -681,19 +682,19 @@ bool LotusChart::readChartName(std::shared_ptr<WPSStream> stream)
 		case 5:
 		{
 			auto *serie=chart->getSerie(id, true);
-			serie->m_legendText=name;
+			serie->m_legendText=uniName;
 			chart->m_hasLegend=true;
 			break;
 		}
 		case 6:
 		case 7:
 		case 8:
-			chart->getAxis(id-6).m_title=name;
+			chart->getAxis(id-6).m_title=uniName;
 			break;
 		case 9:
 		case 10:
 		case 11:
-			chart->getAxis(id-9).m_subTitle=name;
+			chart->getAxis(id-9).m_subTitle=uniName;
 			break;
 		case 12:
 		case 13:
@@ -1014,7 +1015,7 @@ bool LotusChart::readMacPlacement(std::shared_ptr<WPSStream> stream, long endPos
 		f << "title,";
 		if ((val&0x10)==0)
 		{
-			WKSChart::TextZone::Type(allTypes[])= {WKSChart::TextZone::T_Title, WKSChart::TextZone::T_SubTitle};
+			WKSChart::TextZone::Type allTypes[]= {WKSChart::TextZone::T_Title, WKSChart::TextZone::T_SubTitle};
 			for (auto type : allTypes)
 				chart->getTextZone(type,true)->m_show=false;
 		}
@@ -1061,7 +1062,7 @@ bool LotusChart::readMacFloor(std::shared_ptr<WPSStream> stream, long endPos)
 	for (int i=0; i<4; ++i)
 	{
 		auto val=int(libwps::readU8(input));
-		int const(expected[])= {0xf,0x1e,0x12,0};
+		int const expected[]= {0xf,0x1e,0x12,0};
 		if (val!=expected[i])
 			f << "f" << i << "=" << val << ",";
 	}
@@ -1266,7 +1267,7 @@ bool LotusChart::readSerie(std::shared_ptr<WPSStream> stream, long endPos)
 	serie->m_style.m_lineWidth=1;
 	if (format>=0 && format<5)
 	{
-		char const *(wh[])= {"both", "lines", "symbols", "neither", "area"};
+		char const *wh[]= {"both", "lines", "symbols", "neither", "area"};
 		f << "format=" << wh[format] << ",";
 		if (chart->m_fileType<=3 || chart->m_fileType==7)
 		{
@@ -1514,19 +1515,18 @@ bool LotusChart::readSerieName(std::shared_ptr<WPSStream> stream, long endPos)
 	}
 	auto id=int(libwps::readU8(input));
 	f << "id[serie]=" << id << ",";
-	librevenge::RVNGString name("");
-	auto fontType=m_mainParser.getDefaultFontType();
+	std::string name("");
 	for (int i = 0; i < sz-5; i++)
 	{
 		unsigned char c = libwps::readU8(input);
 		if (c == '\0') break;
-		libwps::appendUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType), name);
+		name+=char(c);
 	}
 	if (!name.empty())
 	{
-		f << name.cstr() << ",";
+		f << name << ",";
 		auto *serie=chart->getSerie(id, true);
-		serie->m_legendText=name;
+		serie->m_legendText=libwps_tools_win::Font::unicodeString(name, m_mainParser.getDefaultFontType());
 		chart->m_hasLegend=true;
 	}
 	ascFile.addPos(pos-6);
@@ -1615,7 +1615,7 @@ bool LotusChart::readPlotArea(std::shared_ptr<WPSStream> stream, long endPos)
 		auto val=int(libwps::read16(input));
 		if (val) f << "f" << i << "=" << val << ",";
 	}
-	char const *(zonesName[])= {"title", "note", "serie,legend", "plot"};
+	char const *zonesName[]= {"title", "note", "serie,legend", "plot"};
 	for (int i=0; i<4; ++i)
 	{
 		int dim[4]; // in percent * 65536
@@ -1665,7 +1665,7 @@ bool LotusChart::readPlotArea(std::shared_ptr<WPSStream> stream, long endPos)
 			f << "fl=" << std::hex << val << std::dec << ",";
 		f << "],";
 	}
-	char const *(axisNames[])= {"X","Y","YSecond"};
+	char const *axisNames[]= {"X","Y","YSecond"};
 	for (auto axisName : axisNames) // useme
 	{
 		auto val=int(libwps::readU8(input));
@@ -1769,7 +1769,7 @@ bool LotusChart::readFramesStyle(std::shared_ptr<WPSStream> stream, long endPos)
 	for (int i=0; i<4; ++i)
 	{
 		pos=input->tell();
-		char const *(zonesName[])= {"title", "serie,legend", "note", "plot"};
+		char const *zonesName[]= {"title", "serie,legend", "note", "plot"};
 		f.str("");
 		f << "ChartFramesStyle-" << zonesName[i] << ":";
 		WPSColor color[4]= {WPSColor::black(), WPSColor::white(), WPSColor::black(), WPSColor::black()};
@@ -2007,7 +2007,7 @@ void LotusChart::updateChart(LotusChartInternal::Chart &chart, int id)
 				}
 			}
 			findSerie=true;
-			uint32_t const(defColor[6])= {0xff0000, 0x00ff00, 0x0000ff, 0xffff00, 0x00ffff, 0xff00ff};
+			uint32_t const defColor[6] = {0xff0000, 0x00ff00, 0x0000ff, 0xffff00, 0x00ffff, 0xff00ff};
 			WPSColor color=WPSColor(defColor[i]);
 			if (format.m_color)
 				m_styleManager->getColor256(format.m_color, color);
@@ -2096,7 +2096,7 @@ void LotusChart::updateChart(LotusChartInternal::Chart &chart, int id)
 	if (linkMap.find(name)!=linkMap.end() && convert(linkMap.find(name)->second,ranges))
 	{
 		auto &axis = chart.getAxis(0);
-		for (int r=0; r<2; ++r)	axis.m_cellRanges[r]=ranges[r];
+		for (int r=0; r<2; ++r)	axis.m_labelRanges[r]=ranges[r];
 	}
 	else if (chart.m_fileType==2)
 	{
@@ -2107,7 +2107,7 @@ void LotusChart::updateChart(LotusChartInternal::Chart &chart, int id)
 			auto &axis=chart.getAxis(0);
 			for (int i=0; i<2; ++i)
 			{
-				axis.m_cellRanges[i]=serie->m_ranges[i];
+				axis.m_labelRanges[i]=serie->m_ranges[i];
 				serie->m_ranges[i]=WKSChart::Position();
 			}
 		}
@@ -2127,14 +2127,8 @@ bool LotusChart::sendText(std::shared_ptr<WPSStream> stream, WPSEntry const &ent
 	if (stream.get() == nullptr || !entry.valid())
 		return true;
 	RVNGInputStreamPtr &input = stream->m_input;
-	auto const fontType = m_mainParser.getDefaultFontType();
 	input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
-	for (long i=0; i<entry.length(); ++i)
-	{
-		auto c = (unsigned char) libwps::readU8(input);
-		if (c == '\0') continue;
-		m_listener->insertUnicode((uint32_t)libwps_tools_win::Font::unicode(c,fontType));
-	}
+	m_listener->insertUnicodeString(libwps_tools_win::Font::unicodeString(input.get(), static_cast<unsigned long>(entry.length()), m_mainParser.getDefaultFontType()));
 	return true;
 }
 
