@@ -170,6 +170,8 @@ public:
 		};
 		//! constructor
 		Serie();
+		Serie(Serie const &)=default;
+		Serie &operator=(Serie const &)=default;
 		//! destructor
 		virtual ~Serie();
 		//! return true if the serie style is 1D
@@ -235,6 +237,7 @@ public:
 
 		//! constructor
 		explicit TextZone(Type type);
+		TextZone(TextZone const &)=default;
 		//! destructor
 		~TextZone();
 		//! returns true if the textbox is valid
