@@ -41,6 +41,7 @@
 #include "WPSParagraph.h"
 #include "WPSPosition.h"
 #include "WPSTable.h"
+#include "WKSChart.h"
 #include "WKSSubDocument.h"
 
 ////////////////////////////////////////////////////////////
@@ -62,12 +63,17 @@ struct WKSDocumentParsingState
 	std::map<WPSCellFormat,int,WPSCellFormat::CompareFormat> m_numberingIdMap;
 
 private:
-	WKSDocumentParsingState(const WKSDocumentParsingState &);
-	WKSDocumentParsingState &operator=(const WKSDocumentParsingState &);
+	WKSDocumentParsingState(const WKSDocumentParsingState &) = delete;
+	WKSDocumentParsingState &operator=(const WKSDocumentParsingState &) = delete;
 };
 
-WKSDocumentParsingState::WKSDocumentParsingState(std::vector<WPSPageSpan> const &pageList) :
-	m_pageList(pageList), m_metaData(), m_isDocumentStarted(false), m_isHeaderFooterStarted(false), m_subDocuments(), m_numberingIdMap()
+WKSDocumentParsingState::WKSDocumentParsingState(std::vector<WPSPageSpan> const &pageList)
+	: m_pageList(pageList)
+	, m_metaData()
+	, m_isDocumentStarted(false)
+	, m_isHeaderFooterStarted(false)
+	, m_subDocuments()
+	, m_numberingIdMap()
 {
 }
 
@@ -114,7 +120,6 @@ struct WKSContentParsingState
 
 	bool m_isSheetOpened;
 	bool m_isSheetRowOpened;
-	bool m_isSheetColumnOpened;
 	bool m_isSheetCellOpened;
 
 	bool m_inSubDocument;
@@ -123,29 +128,43 @@ struct WKSContentParsingState
 	libwps::SubDocumentType m_subDocumentType;
 
 private:
-	WKSContentParsingState(const WKSContentParsingState &);
-	WKSContentParsingState &operator=(const WKSContentParsingState &);
+	WKSContentParsingState(const WKSContentParsingState &) = delete;
+	WKSContentParsingState &operator=(const WKSContentParsingState &) = delete;
 };
 
-WKSContentParsingState::WKSContentParsingState() :
-	m_isPageSpanOpened(false), m_isFrameOpened(false), m_currentPage(0), m_numPagesRemainingInSpan(0), m_currentPageNumber(1),
-	m_pageFormLength(11.0),	m_pageFormWidth(8.5f), m_pageFormOrientationIsPortrait(true),
-	m_pageMarginLeft(1.0), m_pageMarginRight(1.0), m_pageMarginTop(1.0), m_pageMarginBottom(1.0),
+WKSContentParsingState::WKSContentParsingState()
+	: m_isPageSpanOpened(false)
+	, m_isFrameOpened(false)
+	, m_currentPage(0)
+	, m_numPagesRemainingInSpan(0)
+	, m_currentPageNumber(1)
+	, m_pageFormLength(11.0)
+	, m_pageFormWidth(8.5)
+	, m_pageFormOrientationIsPortrait(true)
+	, m_pageMarginLeft(1.0)
+	, m_pageMarginRight(1.0)
+	, m_pageMarginTop(1.0)
+	, m_pageMarginBottom(1.0)
 
-	m_textBuffer(""), m_numDeferredTabs(0),
+	, m_textBuffer("")
+	, m_numDeferredTabs(0)
 
-	m_font(), m_paragraph(),
-	m_isGroupOpened(false),
-	m_isParagraphColumnBreak(false), m_isParagraphPageBreak(false),
+	, m_font()
+	, m_paragraph()
+	, m_isGroupOpened(false)
+	, m_isParagraphColumnBreak(false)
+	, m_isParagraphPageBreak(false)
 
-	m_isSpanOpened(false), m_isParagraphOpened(false),
+	, m_isSpanOpened(false)
+	, m_isParagraphOpened(false)
 
-	m_isSheetOpened(false), m_isSheetRowOpened(false), m_isSheetColumnOpened(false),
-	m_isSheetCellOpened(false),
+	, m_isSheetOpened(false)
+	, m_isSheetRowOpened(false)
+	, m_isSheetCellOpened(false)
 
-	m_inSubDocument(false),
-	m_isNote(false),
-	m_subDocumentType(libwps::DOC_NONE)
+	, m_inSubDocument(false)
+	, m_isNote(false)
+	, m_subDocumentType(libwps::DOC_NONE)
 {
 	m_font.m_size=12.0;
 	m_font.m_name="Times New Roman";
@@ -155,9 +174,11 @@ WKSContentParsingState::~WKSContentParsingState()
 {
 }
 
-WKSContentListener::WKSContentListener(std::vector<WPSPageSpan> const &pageList, librevenge::RVNGSpreadsheetInterface *documentInterface) :
-	m_ds(new WKSDocumentParsingState(pageList)), m_ps(new WKSContentParsingState), m_psStack(),
-	m_documentInterface(documentInterface)
+WKSContentListener::WKSContentListener(std::vector<WPSPageSpan> const &pageList, librevenge::RVNGSpreadsheetInterface *documentInterface)
+	: m_ds(new WKSDocumentParsingState(pageList))
+	, m_ps(new WKSContentParsingState)
+	, m_psStack()
+	, m_documentInterface(documentInterface)
 {
 }
 
@@ -552,6 +573,36 @@ void WKSContentListener::insertComment(WPSSubDocumentPtr &subDocument)
 	m_ps->m_isNote = false;
 }
 
+///////////////////
+// chart
+///////////////////
+void WKSContentListener::insertChart
+(WPSPosition const &pos, WKSChart const &chart, WPSGraphicStyle const &style)
+{
+	WPSGraphicStyle finalStyle(style);
+	if (!chart.m_name.empty())
+		finalStyle.m_frameName=chart.m_name;
+	if (!_openFrame(pos, finalStyle)) return;
+
+	_pushParsingState();
+	_startSubDocument();
+	m_ps->m_subDocumentType = libwps::DOC_CHART_ZONE;
+
+	std::shared_ptr<WKSContentListener> listen(this, WPS_shared_ptr_noop_deleter<WKSContentListener>());
+	try
+	{
+		chart.sendChart(listen, m_documentInterface);
+	}
+	catch (...)
+	{
+		WPS_DEBUG_MSG(("WKSContentListener::insertChart exception catched \n"));
+	}
+	_endSubDocument();
+	_popParsingState();
+
+	_closeFrame();
+}
+
 void WKSContentListener::insertTextBox
 (WPSPosition const &pos, WPSSubDocumentPtr subDocument, WPSGraphicStyle const &frameStyle)
 {
@@ -699,6 +750,20 @@ bool WKSContentListener::_openFrame(WPSPosition const &pos, WPSGraphicStyle cons
 		else
 			_openSpan();
 		break;
+	case WPSPosition::Cell:
+		if (!m_ps->m_isSheetCellOpened)
+		{
+			WPS_DEBUG_MSG(("WPSSpreadsheetListener::openFrame: called with Cell position not in a sheet cell\n"));
+			return false;
+		}
+		if (pos.m_anchorCellName.empty())
+		{
+			WPS_DEBUG_MSG(("WPSSpreadsheetListener::openFrame: can not find the cell name\n"));
+			return false;
+		}
+		if (m_ps->m_isParagraphOpened)
+			_closeParagraph();
+		break;
 	default:
 		WPS_DEBUG_MSG(("WKSContentListener::openFrame: can not determine the anchor\n"));
 		return false;
@@ -738,8 +803,8 @@ void WKSContentListener::_handleFrameParameters
 	propList.insert("svg:height", double(pos.size()[1]), unit);
 	if (pos.naturalSize().x() > 4*pointFactor && pos.naturalSize().y() > 4*pointFactor)
 	{
-		propList.insert("librevenge:naturalWidth", pos.naturalSize().x(), pos.unit());
-		propList.insert("librevenge:naturalHeight", pos.naturalSize().y(), pos.unit());
+		propList.insert("librevenge:naturalWidth", double(pos.naturalSize().x()), pos.unit());
+		propList.insert("librevenge:naturalHeight", double(pos.naturalSize().y()), pos.unit());
 	}
 
 	if (pos.m_wrapping ==  WPSPosition::WDynamic)
@@ -752,6 +817,18 @@ void WKSContentListener::_handleFrameParameters
 	else
 		propList.insert("style:wrap", "none");
 
+	if (pos.m_anchorTo == WPSPosition::Cell)
+	{
+		if (!pos.m_anchorCellName.empty())
+			propList.insert("table:end-cell-address", pos.m_anchorCellName);
+		// todo: implement also different m_xPos and m_yPos
+		if (origin[0] < 0 || origin[0] > 0)
+			propList.insert("svg:x", double(origin[0]), unit);
+		if (origin[1] < 0 || origin[1] > 0)
+			propList.insert("svg:y", double(origin[1]), unit);
+		return;
+	}
+
 	if (pos.m_anchorTo != WPSPosition::Page && pos.m_anchorTo != WPSPosition::PageContent)
 	{
 		WPS_DEBUG_MSG(("WKSContentListener::openFrame: only implemented for page anchor\n"));
@@ -761,8 +838,8 @@ void WKSContentListener::_handleFrameParameters
 	// Page position seems to do not use the page margin...
 	propList.insert("text:anchor-type", "page");
 	if (pos.page() > 0) propList.insert("text:anchor-page-number", pos.page());
-	double  w = m_ps->m_pageFormWidth;
-	double h = m_ps->m_pageFormLength;
+	float w = float(m_ps->m_pageFormWidth);
+	float h = float(m_ps->m_pageFormLength);
 	w *= inchFactor;
 	h *= inchFactor;
 
@@ -770,29 +847,27 @@ void WKSContentListener::_handleFrameParameters
 	propList.insert("style:vertical-rel", relPos);
 	propList.insert("style:horizontal-rel", relPos);
 
-	double newPosition;
+	float newPosition;
 	switch (pos.m_yPos)
 	{
 	case WPSPosition::YFull:
 		propList.insert("svg:height", double(h), unit);
 		WPS_FALLTHROUGH;
 	case WPSPosition::YTop:
-		if (origin[1] < 0.0 || origin[1] > 0.0)
+		if (origin[1] < 0.0f || origin[1] > 0.0f)
 		{
 			propList.insert("style:vertical-pos", "from-top");
 			newPosition = origin[1];
-			if (newPosition > h -pos.size()[1])
-				newPosition = h - pos.size()[1];
 			propList.insert("svg:y", double(newPosition), unit);
 		}
 		else
 			propList.insert("style:vertical-pos", "top");
 		break;
 	case WPSPosition::YCenter:
-		if (origin[1] < 0.0 || origin[1] > 0.0)
+		if (origin[1] < 0.0f || origin[1] > 0.0f)
 		{
 			propList.insert("style:vertical-pos", "from-top");
-			newPosition = (h - pos.size()[1])/2.0;
+			newPosition = (h - pos.size()[1])/2.0f;
 			if (newPosition > h -pos.size()[1]) newPosition = h - pos.size()[1];
 			propList.insert("svg:y", double(newPosition), unit);
 		}
@@ -800,7 +875,7 @@ void WKSContentListener::_handleFrameParameters
 			propList.insert("style:vertical-pos", "middle");
 		break;
 	case WPSPosition::YBottom:
-		if (origin[1] < 0.0 || origin[1] > 0.0)
+		if (origin[1] < 0.0f || origin[1] > 0.0f)
 		{
 			propList.insert("style:vertical-pos", "from-top");
 			newPosition = h - pos.size()[1]-origin[1];
@@ -821,7 +896,7 @@ void WKSContentListener::_handleFrameParameters
 		propList.insert("svg:width", double(w), unit);
 		WPS_FALLTHROUGH;
 	case WPSPosition::XLeft:
-		if (origin[0] < 0.0 || origin[0] > 0.0)
+		if (origin[0] < 0.0f || origin[0] > 0.0f)
 		{
 			propList.insert("style:horizontal-pos", "from-left");
 			propList.insert("svg:x", double(origin[0]), unit);
@@ -830,7 +905,7 @@ void WKSContentListener::_handleFrameParameters
 			propList.insert("style:horizontal-pos", "left");
 		break;
 	case WPSPosition::XRight:
-		if (origin[0] < 0.0 || origin[0] > 0.0)
+		if (origin[0] < 0.0f || origin[0] > 0.0f)
 		{
 			propList.insert("style:horizontal-pos", "from-left");
 			propList.insert("svg:x",double(w - pos.size()[0] + origin[0]), unit);
@@ -840,10 +915,10 @@ void WKSContentListener::_handleFrameParameters
 		break;
 	case WPSPosition::XCenter:
 	default:
-		if (origin[0] < 0.0 || origin[0] > 0.0)
+		if (origin[0] < 0.0f || origin[0] > 0.0f)
 		{
 			propList.insert("style:horizontal-pos", "from-left");
-			propList.insert("svg:x", double((w - pos.size()[0])/2. + origin[0]), unit);
+			propList.insert("svg:x", double((w - pos.size()[0])/2.f + origin[0]), unit);
 		}
 		else
 			propList.insert("style:horizontal-pos", "center");
@@ -864,11 +939,13 @@ void WKSContentListener::handleSubDocument(WPSSubDocumentPtr &subDocument, libwp
 
 	// Check whether the document is calling itself
 	bool sendDoc = true;
-	for (size_t i = 0; i < m_ds->m_subDocuments.size(); i++)
+	for (auto subDoc : m_ds->m_subDocuments)
 	{
 		if (!subDocument)
 			break;
-		if (subDocument == m_ds->m_subDocuments[i])
+		if (!subDoc)
+			continue;
+		if (*subDocument == subDoc)
 		{
 			WPS_DEBUG_MSG(("WKSContentListener::handleSubDocument: recursif call, stop...\n"));
 			sendDoc = false;
@@ -880,10 +957,10 @@ void WKSContentListener::handleSubDocument(WPSSubDocumentPtr &subDocument, libwp
 		if (subDocument)
 		{
 			m_ds->m_subDocuments.push_back(subDocument);
-			shared_ptr<WKSContentListener> listen(this, WPS_shared_ptr_noop_deleter<WKSContentListener>());
+			std::shared_ptr<WKSContentListener> listen(this, WPS_shared_ptr_noop_deleter<WKSContentListener>());
 			try
 			{
-				WKSSubDocument *subDoc=dynamic_cast<WKSSubDocument *>(subDocument.get());
+				auto *subDoc=dynamic_cast<WKSSubDocument *>(subDocument.get());
 				if (subDoc)
 					subDoc->parse(listen, subDocumentType);
 				else
@@ -941,10 +1018,10 @@ void WKSContentListener::openSheet(std::vector<WPSColumnFormat> const &colList,
 
 	librevenge::RVNGPropertyListVector columns;
 
-	for (size_t c = 0; c < colList.size(); c++)
+	for (auto const &col : colList)
 	{
 		librevenge::RVNGPropertyList column;
-		colList[c].addTo(column);
+		col.addTo(column);
 		columns.append(column);
 	}
 	propList.insert("librevenge:columns", columns);
@@ -1029,7 +1106,7 @@ void WKSContentListener::openSheetCell(WPSCell const &cell, WKSContentListener::
 		}
 		else
 		{
-			numberingId=(int) m_ds->m_numberingIdMap.size();
+			numberingId=int(m_ds->m_numberingIdMap.size());
 			name << "Numbering" << numberingId;
 
 			librevenge::RVNGPropertyList numList;
@@ -1049,8 +1126,8 @@ void WKSContentListener::openSheetCell(WPSCell const &cell, WKSContentListener::
 	if (content.m_formula.size())
 	{
 		librevenge::RVNGPropertyListVector formulaVect;
-		for (size_t i=0; i < content.m_formula.size(); ++i)
-			formulaVect.append(content.m_formula[i].getPropertyList());
+		for (auto const &form : content.m_formula)
+			formulaVect.append(form.getPropertyList());
 		propList.insert("librevenge:formula", formulaVect);
 	}
 	bool hasFormula=!content.m_formula.empty();
@@ -1144,10 +1221,10 @@ void WKSContentListener::_openPageSpan()
 		throw libwps::ParseException();
 	}
 	unsigned actPage = 0;
-	std::vector<WPSPageSpan>::iterator it = m_ds->m_pageList.begin();
+	auto it = m_ds->m_pageList.begin();
 	while (actPage < m_ps->m_currentPage)
 	{
-		actPage+=(unsigned) it++->getPageSpan();
+		actPage+=unsigned(it++->getPageSpan());
 		if (it == m_ds->m_pageList.end())
 		{
 			WPS_DEBUG_MSG(("WKSContentListener::_openPageSpan: can not find current page\n"));
@@ -1199,9 +1276,9 @@ void WKSContentListener::_closePageSpan()
 ///////////////////
 
 // ---------- state stack ------------------
-shared_ptr<WKSContentParsingState> WKSContentListener::_pushParsingState()
+std::shared_ptr<WKSContentParsingState> WKSContentListener::_pushParsingState()
 {
-	shared_ptr<WKSContentParsingState> actual = m_ps;
+	auto actual = m_ps;
 	m_psStack.push_back(actual);
 	m_ps.reset(new WKSContentParsingState);
 
@@ -1259,25 +1336,31 @@ librevenge::RVNGPropertyList WKSContentListener::FormulaInstruction::getProperty
 		break;
 	case F_Cell:
 		pList.insert("librevenge:type","librevenge-cell");
-		pList.insert("librevenge:column",m_position[0][0], librevenge::RVNG_GENERIC);
-		pList.insert("librevenge:row",m_position[0][1], librevenge::RVNG_GENERIC);
+		pList.insert("librevenge:column",m_position[0][0]);
+		pList.insert("librevenge:row",m_position[0][1]);
 		pList.insert("librevenge:column-absolute",!m_positionRelative[0][0]);
 		pList.insert("librevenge:row-absolute",!m_positionRelative[0][1]);
-		if (!m_sheetName.empty())
-			pList.insert("librevenge:sheet-name",m_sheetName.cstr());
+		if (!m_sheetName[0].empty())
+			pList.insert("librevenge:sheet-name",m_sheetName[0].cstr());
+		if (!m_fileName.empty())
+			pList.insert("librevenge:file-name",m_fileName.cstr());
 		break;
 	case F_CellList:
 		pList.insert("librevenge:type","librevenge-cells");
-		pList.insert("librevenge:start-column",m_position[0][0], librevenge::RVNG_GENERIC);
-		pList.insert("librevenge:start-row",m_position[0][1], librevenge::RVNG_GENERIC);
+		pList.insert("librevenge:start-column",m_position[0][0]);
+		pList.insert("librevenge:start-row",m_position[0][1]);
 		pList.insert("librevenge:start-column-absolute",!m_positionRelative[0][0]);
 		pList.insert("librevenge:start-row-absolute",!m_positionRelative[0][1]);
-		pList.insert("librevenge:end-column",m_position[1][0], librevenge::RVNG_GENERIC);
-		pList.insert("librevenge:end-row",m_position[1][1], librevenge::RVNG_GENERIC);
+		pList.insert("librevenge:end-column",m_position[1][0]);
+		pList.insert("librevenge:end-row",m_position[1][1]);
 		pList.insert("librevenge:end-column-absolute",!m_positionRelative[1][0]);
 		pList.insert("librevenge:end-row-absolute",!m_positionRelative[1][1]);
-		if (!m_sheetName.empty())
-			pList.insert("librevenge:sheet-name",m_sheetName.cstr());
+		if (!m_sheetName[0].empty())
+			pList.insert("librevenge:sheet-name",m_sheetName[0].cstr());
+		if (!m_sheetName[1].empty())
+			pList.insert("librevenge:end-sheet-name",m_sheetName[1].cstr());
+		if (!m_fileName.empty())
+			pList.insert("librevenge:file-name",m_fileName.cstr());
 		break;
 	default:
 		WPS_DEBUG_MSG(("WKSContentListener::FormulaInstruction::getPropertyList: unexpected type\n"));
@@ -1293,27 +1376,27 @@ std::ostream &operator<<(std::ostream &o, WKSContentListener::FormulaInstruction
 		o << inst.m_longValue;
 	else if (inst.m_type==WKSContentListener::FormulaInstruction::F_Cell)
 	{
-		if (!inst.m_positionRelative[0][0]) o << "$";
-		if (inst.m_position[0][0]>=26) o << (char)(inst.m_position[0][0]/26-1 + 'A');
-		o << (char)(inst.m_position[0][0]%26+'A');
-		if (!inst.m_positionRelative[0][1]) o << "$";
-		o << inst.m_position[0][1];
-		if (!inst.m_sheetName.empty())
-			o << "[" << inst.m_sheetName.cstr() << "]";
+		o << libwps::getCellName(inst.m_position[0],inst.m_positionRelative[0]);
+		if (!inst.m_sheetName[0].empty())
+			o << "[" << inst.m_sheetName[0].cstr() << "]";
+		else if (inst.m_sheetId[0]>=0)
+			o << "[sheet" << inst.m_sheetId[0] << "]";
+		if (!inst.m_fileName.empty())
+			o << "[file=" << inst.m_fileName.cstr() << "]";
 	}
 	else if (inst.m_type==WKSContentListener::FormulaInstruction::F_CellList)
 	{
 		for (int l=0; l<2; ++l)
 		{
-			if (!inst.m_positionRelative[l][0]) o << "$";
-			if (inst.m_position[l][0]>=26) o << (char)(inst.m_position[l][0]/26-1 + 'A');
-			o << (char)(inst.m_position[l][0]%26+'A');
-			if (!inst.m_positionRelative[l][1]) o << "$";
-			o << inst.m_position[l][1];
+			o << libwps::getCellName(inst.m_position[l],inst.m_positionRelative[l]);
+			if (!inst.m_sheetName[l].empty())
+				o << "[" << inst.m_sheetName[l].cstr() << "]";
+			else if (inst.m_sheetId[l]>=0)
+				o << "[sheet" << inst.m_sheetId[l] << "]";
 			if (l==0) o << ":";
 		}
-		if (!inst.m_sheetName.empty())
-			o << "[" << inst.m_sheetName.cstr() << "]";
+		if (!inst.m_fileName.empty())
+			o << "[file=" << inst.m_fileName.cstr() << "]";
 	}
 	else if (inst.m_type==WKSContentListener::FormulaInstruction::F_Text)
 		o << "\"" << inst.m_content << "\"";
@@ -1326,7 +1409,7 @@ std::ostream &operator<<(std::ostream &o, WKSContentListener::FormulaInstruction
 bool WKSContentListener::CellContent::double2Date(double val, int &Y, int &M, int &D)
 {
 	/* first convert the date in long. Checkme: unsure why -2 is needed...*/
-	long numDaysSinceOrigin=long(val-2+0.4);
+	auto numDaysSinceOrigin=long(val-2+0.4);
 	// checkme: do we need to check before for isNan(val) ?
 	if (numDaysSinceOrigin<-10000*365 || numDaysSinceOrigin>10000*365)
 	{
@@ -1361,7 +1444,7 @@ bool WKSContentListener::CellContent::double2Date(double val, int &Y, int &M, in
 		numDaysToEndY1=Y*365+(Y>0 ? (Y-1)/4+((century%4)?0:1) : 0);
 	}
 	// finish to compute the date
-	int numDaysFrom1Jan=int(numDaysSinceOrigin-numDaysToEndY1);
+	auto numDaysFrom1Jan=int(numDaysSinceOrigin-numDaysToEndY1);
 	Y+=century*100;
 	bool isLeap=(Y%4)==0 && ((Y%400)==0 || (Y%100)!=0);
 
@@ -1417,8 +1500,8 @@ std::ostream &operator<<(std::ostream &o, WKSContentListener::CellContent const
 	break;
 	case WKSContentListener::CellContent::C_FORMULA:
 		o << ",formula=";
-		for (size_t l=0; l < cell.m_formula.size(); ++l)
-			o << cell.m_formula[l];
+		for (const auto &l : cell.m_formula)
+			o << l;
 		if (cell.isValueSet()) o << "[" << cell.m_value << "]";
 		break;
 	case WKSContentListener::CellContent::C_UNKNOWN:
