@@ -42,34 +42,34 @@
 namespace WPS4ParserInternal
 {
 //! Internal: the subdocument of a WPS4Parser
-class SubDocument : public WPSTextSubDocument
+class SubDocument final : public WPSTextSubDocument
 {
 public:
 	//! type of an entry stored in textId
 	enum Type { Unknown, MN };
 	//! constructor for a text entry
-	SubDocument(RVNGInputStreamPtr input, WPS4Parser &pars, WPSEntry const &entry) :
-		WPSTextSubDocument(input, &pars), m_entry(entry) {}
+	SubDocument(RVNGInputStreamPtr const &input, WPS4Parser &pars, WPSEntry const &entry)
+		: WPSTextSubDocument(input, &pars), m_entry(entry) {}
 	//! destructor
-	~SubDocument() {}
+	~SubDocument() final {}
 
 	//! operator==
-	virtual bool operator==(shared_ptr<WPSTextSubDocument> const &doc) const
+	bool operator==(std::shared_ptr<WPSSubDocument> const &doc) const final
 	{
 		if (!doc || !WPSTextSubDocument::operator==(doc))
 			return false;
-		SubDocument const *sDoc = dynamic_cast<SubDocument const *>(doc.get());
+		auto const *sDoc = dynamic_cast<SubDocument const *>(doc.get());
 		if (!sDoc) return false;
 		return m_entry == sDoc->m_entry;
 	}
 
 	//! the parser function
-	void parse(shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType);
+	void parse(std::shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType) final;
 	//! the entry
 	WPSEntry m_entry;
 };
 
-void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType)
+void SubDocument::parse(std::shared_ptr<WPSContentListener> &listener, libwps::SubDocumentType subDocumentType)
 {
 	if (!listener.get())
 	{
@@ -82,7 +82,7 @@ void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDoc
 		return;
 	}
 
-	WPSContentListenerPtr &listen =  reinterpret_cast<WPSContentListenerPtr &>(listener);
+	auto &listen =  reinterpret_cast<WPSContentListenerPtr &>(listener);
 	if (!m_parser)
 	{
 		listen->insertCharacter(' ');
@@ -114,7 +114,7 @@ void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDoc
 		return;
 	}
 
-	WPS4Parser *mnParser = dynamic_cast<WPS4Parser *>(m_parser);
+	auto *mnParser = dynamic_cast<WPS4Parser *>(m_parser);
 	if (!mnParser)
 	{
 		WPS_DEBUG_MSG(("SubDocument::parse: bad parser...\n"));
@@ -127,11 +127,17 @@ void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDoc
 //! the state of WPS4
 struct State
 {
-	explicit State(libwps_tools_win::Font::Type fontType) :
-		m_isDosFile(false), m_eof(-1), m_fontType(fontType),
-		m_OEMFontType(libwps_tools_win::Font::UNKNOWN),
-		m_pageSpan(), m_noFirstPageHeader(false), m_noFirstPageFooter(false),
-		m_numColumns(1), m_actPage(0), m_numPages(0)
+	explicit State(libwps_tools_win::Font::Type fontType)
+		: m_isDosFile(false)
+		, m_eof(-1)
+		, m_fontType(fontType)
+		, m_OEMFontType(libwps_tools_win::Font::UNKNOWN)
+		, m_pageSpan()
+		, m_noFirstPageHeader(false)
+		, m_noFirstPageFooter(false)
+		, m_numColumns(1)
+		, m_actPage(0)
+		, m_numPages(0)
 	{
 	}
 	//! flag to know if the file is or not a dos file
@@ -153,9 +159,12 @@ struct State
 
 // constructor, destructor
 WPS4Parser::WPS4Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header,
-                       libwps_tools_win::Font::Type encoding) :
-	WPSParser(input, header),
-	m_listener(), m_graphParser(), m_textParser(), m_state()
+                       libwps_tools_win::Font::Type encoding)
+	: WPSParser(input, header)
+	, m_listener()
+	, m_graphParser()
+	, m_textParser()
+	, m_state()
 {
 	m_state.reset(new WPS4ParserInternal::State(encoding));
 	m_graphParser.reset(new WPS4Graph(*this));
@@ -250,14 +259,14 @@ bool WPS4Parser::checkFilePosition(long pos)
 }
 
 // listener, new page
-void WPS4Parser::setListener(shared_ptr<WPSContentListener> listener)
+void WPS4Parser::setListener(std::shared_ptr<WPSContentListener> const &listener)
 {
 	m_listener = listener;
 	m_graphParser->setListener(m_listener);
 	m_textParser->setListener(m_listener);
 }
 
-shared_ptr<WPSContentListener> WPS4Parser::createListener(librevenge::RVNGTextInterface *interface)
+std::shared_ptr<WPSContentListener> WPS4Parser::createListener(librevenge::RVNGTextInterface *interface)
 {
 	std::vector<WPSPageSpan> pageList;
 	WPSPageSpan page1(m_state->m_pageSpan), ps(m_state->m_pageSpan);
@@ -291,7 +300,7 @@ shared_ptr<WPSContentListener> WPS4Parser::createListener(librevenge::RVNGTextIn
 	pageList.push_back(page1);
 	for (int i = 1; i < numPages; i++) pageList.push_back(ps);
 	m_state->m_numPages=numPages;
-	return shared_ptr<WPSContentListener>
+	return std::shared_ptr<WPSContentListener>
 	       (new WPSContentListener(pageList, interface));
 }
 
@@ -304,7 +313,7 @@ void WPS4Parser::newPage(int number)
 	{
 		m_state->m_actPage++;
 
-		if (m_listener.get() == 0L || m_state->m_actPage == 1)
+		if (m_listener.get() == nullptr || m_state->m_actPage == 1)
 			continue;
 		m_listener->insertBreak(WPS_PAGE_BREAK);
 		m_graphParser->sendObjects(m_state->m_actPage);
@@ -343,20 +352,20 @@ void WPS4Parser::send(WPSEntry const &entry, libwps::SubDocumentType)
 
 void WPS4Parser::createDocument(WPSEntry const &entry, libwps::SubDocumentType type)
 {
-	if (m_listener.get() == 0L) return;
+	if (m_listener.get() == nullptr) return;
 	WPSSubDocumentPtr subdoc(new WPS4ParserInternal::SubDocument
 	                         (getInput(), *this, entry));
 	if (type == libwps::DOC_COMMENT_ANNOTATION)
 		m_listener->insertComment(subdoc);
 	else
 	{
-		WPS_DEBUG_MSG(("WPS4Parser:createDocument error: unknown type: \"%d\"\n", (int) type));
+		WPS_DEBUG_MSG(("WPS4Parser:createDocument error: unknown type: \"%d\"\n", int(type)));
 	}
 }
 
 void WPS4Parser::createNote(WPSEntry const &entry, librevenge::RVNGString const &label)
 {
-	if (m_listener.get() == 0L) return;
+	if (m_listener.get() == nullptr) return;
 	WPSSubDocumentPtr subdoc(new WPS4ParserInternal::SubDocument
 	                         (getInput(), *this, entry));
 	m_listener->insertLabelNote(WPSContentListener::FOOTNOTE, label, subdoc);
@@ -364,7 +373,7 @@ void WPS4Parser::createNote(WPSEntry const &entry, librevenge::RVNGString const
 
 void WPS4Parser::createTextBox(WPSEntry const &entry, WPSPosition const &pos, librevenge::RVNGPropertyList &extras)
 {
-	if (m_listener.get() == 0L) return;
+	if (m_listener.get() == nullptr) return;
 	WPSSubDocumentPtr subdoc(new WPS4ParserInternal::SubDocument
 	                         (getInput(), *this, entry));
 	m_listener->insertTextBox(pos, subdoc, extras);
@@ -383,7 +392,7 @@ void WPS4Parser::parse(librevenge::RVNGTextInterface *documentInterface)
 		WPS_DEBUG_MSG(("WPS4Parser::parse: does not find main ole\n"));
 		throw (libwps::ParseException());
 	}
-	if (!checkHeader(0, true))
+	if (!checkHeader(nullptr, true))
 		throw (libwps::ParseException());
 	try
 	{
@@ -456,20 +465,17 @@ bool WPS4Parser::createOLEStructures()
 	if (!input) return false;
 
 	if (!input->isStructured()) return true;
-	WPSOLEParser oleParser("MN0");
+	WPSOLEParser oleParser("MN0", getDefaultFontType());
 	if (!oleParser.parse(input)) return false;
 
-	m_graphParser->storeObjects(oleParser.getObjects(),oleParser.getObjectsId());
+	m_graphParser->storeObjects(oleParser.getObjectsMap());
 #ifdef DEBUG
 	// there can remain some embedded Works subdocument ( WKS, ... )
 	// with name MN0 and some unknown picture ole
-	std::vector<std::string> unparsed = oleParser.getNotParse();
+	auto unparsed = oleParser.getNotParse();
 
-	size_t numUnparsed = unparsed.size();
-
-	for (size_t i = 0; i < numUnparsed; i++)
+	for (auto const &name : unparsed)
 	{
-		std::string const &name = unparsed[i];
 		if (name == "MN0")
 			continue;
 		WPS_DEBUG_MSG(("WPS4Parser::createOLEStructures:: Find unparsed ole: %s\n", name.c_str()));
@@ -505,7 +511,7 @@ bool WPS4Parser::parseEntry(std::string const &name)
 	RVNGInputStreamPtr input = getInput();
 	long actPos = input->tell();
 	WPSEntry zone;
-	zone.setBegin((long) libwps::readU32(input));
+	zone.setBegin(long(libwps::readU32(input)));
 	zone.setLength(libwps::readU16(input));
 	zone.setType(name);
 
@@ -637,10 +643,10 @@ bool WPS4Parser::findZones()
 	for (int i = 0; i < 2; i++)
 		f << libwps::readU16(input)/1440. << ",";
 	f << "),";
-	int oem=(int) libwps::readU16(input);
+	auto oem=int(libwps::readU16(input));
 	if ((oem>>4)&0x7ff)
 	{
-		libwps_tools_win::Font::Type type=libwps_tools_win::Font::getTypeForOEM((oem>>4)&0x7ff);
+		auto type=libwps_tools_win::Font::getTypeForOEM((oem>>4)&0x7ff);
 		if (type != libwps_tools_win::Font::UNKNOWN)
 		{
 			m_state->m_OEMFontType = type;
@@ -700,7 +706,7 @@ bool WPS4Parser::findZones()
 	val = libwps::readU16(input);
 	if (val & 0xFF00) f << "#unkn=" << (val >> 8) << ",";
 	val &= 0xFF;
-	int numCols = int(val/2);
+	auto numCols = int(val/2);
 	if (numCols >= 1 && numCols <= 13)
 	{
 		if (version() >= 3)
@@ -738,7 +744,7 @@ bool WPS4Parser::findZones()
 	f.str("");
 	f << "ZZHeader-II:";
 	bool empty = true;
-	long begP = (long) libwps::readU32(input);
+	auto begP = long(libwps::readU32(input));
 	if (begP)
 	{
 		if (begP <= 0 || !checkFilePosition(begP)) f << "###";
@@ -753,7 +759,7 @@ bool WPS4Parser::findZones()
 
 	int num = libwps::readU16(input);
 	long sz = libwps::readU16(input);
-	begP = (long) libwps::readU32(input);
+	begP = long(libwps::readU32(input));
 	if (begP)
 	{
 		if (begP <= 0 || !checkFilePosition(begP)) f << "###";
@@ -805,7 +811,7 @@ bool WPS4Parser::findZones()
 	int i = 0;
 	while (input->tell()+4 <= 0xd0)
 	{
-		val = (long) libwps::readU32(input);
+		val = long(libwps::readU32(input));
 		if (val)
 		{
 			f << "f" << std::dec << i << "=" << std::hex << val << ",";
@@ -840,8 +846,8 @@ bool WPS4Parser::readDocDim()
 	libwps::DebugStream f;
 	f << "Entries(DocDim):";
 	double margin[4], size[2];
-	for (int i = 0; i < 4; i++) margin[i] = libwps::readU16(input)/1440.;
-	for (int i = 0; i < 2; i++) size[i] = libwps::readU16(input)/1440.;
+	for (double &i : margin) i = libwps::readU16(input)/1440.;
+	for (double &i : size) i = libwps::readU16(input)/1440.;
 
 	if (margin[0]+margin[1] > size[0] || margin[2]+margin[3] > size[1])
 	{
@@ -858,7 +864,7 @@ bool WPS4Parser::readDocDim()
 	page.setFormWidth(size[1]);
 
 	int unkns[8];
-	for (int i = 0; i < 8; i++) unkns[i] = libwps::readU16(input);
+	for (int &unkn : unkns) unkn = libwps::readU16(input);
 	int page_orientation = unkns[5];
 	if (page_orientation == 0)
 		page.setFormOrientation(WPSPageSpan::PORTRAIT);
@@ -962,7 +968,7 @@ bool WPS4Parser::readPrnt(WPSEntry const &entry)
 		f.str("");
 		f << "ZZPRNT(II):";
 		for (int i = 0; i < length; i++)
-			f << (char) libwps::readU8(input);
+			f << char(libwps::readU8(input));
 		ascii().addNote(f.str().c_str());
 	}
 	return true;
@@ -989,7 +995,7 @@ bool WPS4Parser::readDocWindowsInfo(WPSEntry const &entry)
 	// Find in one file str[0]="C:\Databases\Elem 02-03.wdb",str[120]="Query1"
 	while (pos++ < 0x132)   // try to find some strings
 	{
-		char c = char(libwps::read8(input));
+		auto c = char(libwps::read8(input));
 		if (c == '\0')
 		{
 			if (debSPos>=0)
@@ -1021,7 +1027,7 @@ bool WPS4Parser::readDocWindowsInfo(WPSEntry const &entry)
 	}
 
 	int dim[2]; // TB, LR
-	for (int i = 0; i < 2; i++) dim[i] = libwps::read16(input);
+	for (int &i : dim) i = libwps::read16(input);
 	if (dim[0] || dim[1])
 		f << "pageBorderDist=" << dim[1]/1440. << "x" << dim[0]/1440. << ",";
 	long val = libwps::readU8(input);
@@ -1033,7 +1039,7 @@ bool WPS4Parser::readDocWindowsInfo(WPSEntry const &entry)
 	val = libwps::readU8(input);
 	if (val)
 		f << "pageBorderStyle=" << val << ",";
-	val = (long) libwps::readU32(input);
+	val = long(libwps::readU32(input));
 	if (val&2)
 		f << "pageBorderShaded,";
 	if (val&1)
