@@ -60,15 +60,19 @@ struct WPSDocumentParsingState
 	std::vector<WPSSubDocumentPtr> m_subDocuments; /** list of document actually open */
 
 private:
-	WPSDocumentParsingState(const WPSDocumentParsingState &);
-	WPSDocumentParsingState &operator=(const WPSDocumentParsingState &);
+	WPSDocumentParsingState(const WPSDocumentParsingState &) = delete;
+	WPSDocumentParsingState &operator=(const WPSDocumentParsingState &) = delete;
 };
 
-WPSDocumentParsingState::WPSDocumentParsingState(std::vector<WPSPageSpan> const &pageList) :
-	m_pageList(pageList),
-	m_metaData(),
-	m_footNoteNumber(0), m_endNoteNumber(0), m_newListId(0),
-	m_isDocumentStarted(false), m_isHeaderFooterStarted(false), m_subDocuments()
+WPSDocumentParsingState::WPSDocumentParsingState(std::vector<WPSPageSpan> const &pageList)
+	: m_pageList(pageList)
+	, m_metaData()
+	, m_footNoteNumber(0)
+	, m_endNoteNumber(0)
+	, m_newListId(0)
+	, m_isDocumentStarted(false)
+	, m_isHeaderFooterStarted(false)
+	, m_subDocuments()
 {
 }
 
@@ -88,7 +92,7 @@ struct WPSContentParsingState
 
 	WPSFont m_font;
 	WPSParagraph m_paragraph;
-	shared_ptr<WPSList> m_list;
+	std::shared_ptr<WPSList> m_list;
 
 	bool m_isParagraphColumnBreak;
 	bool m_isParagraphPageBreak;
@@ -139,50 +143,62 @@ struct WPSContentParsingState
 	libwps::SubDocumentType m_subDocumentType;
 
 private:
-	WPSContentParsingState(const WPSContentParsingState &);
-	WPSContentParsingState &operator=(const WPSContentParsingState &);
+	WPSContentParsingState(const WPSContentParsingState &) = delete;
+	WPSContentParsingState &operator=(const WPSContentParsingState &) = delete;
 };
 
-WPSContentParsingState::WPSContentParsingState() :
-	m_textBuffer(""), m_numDeferredTabs(0),
+WPSContentParsingState::WPSContentParsingState()
+	: m_textBuffer("")
+	, m_numDeferredTabs(0)
 
-	m_font(), m_paragraph(), m_list(),
-	m_isParagraphColumnBreak(false), m_isParagraphPageBreak(false),
+	, m_font()
+	, m_paragraph()
+	, m_list()
+	, m_isParagraphColumnBreak(false)
+	, m_isParagraphPageBreak(false)
 
-	m_isPageSpanOpened(false), m_isSectionOpened(false), m_isFrameOpened(false),
-	m_isPageSpanBreakDeferred(false),
-	m_isHeaderFooterWithoutParagraph(false),
-	m_isGroupOpened(false),
+	, m_isPageSpanOpened(false)
+	, m_isSectionOpened(false)
+	, m_isFrameOpened(false)
+	, m_isPageSpanBreakDeferred(false)
+	, m_isHeaderFooterWithoutParagraph(false)
+	, m_isGroupOpened(false)
 
-	m_isSpanOpened(false), m_isParagraphOpened(false), m_isListElementOpened(false),
+	, m_isSpanOpened(false)
+	, m_isParagraphOpened(false)
+	, m_isListElementOpened(false)
 
-	m_firstParagraphInPageSpan(true),
+	, m_firstParagraphInPageSpan(true)
 
-	m_numRowsToSkip(),
-	m_isTableOpened(false), m_isTableRowOpened(false), m_isTableColumnOpened(false),
-	m_isTableCellOpened(false),
+	, m_numRowsToSkip()
+	, m_isTableOpened(false)
+	, m_isTableRowOpened(false)
+	, m_isTableColumnOpened(false)
+	, m_isTableCellOpened(false)
 
-	m_currentPage(0), m_numPagesRemainingInSpan(0), m_currentPageNumber(1),
+	, m_currentPage(0)
+	, m_numPagesRemainingInSpan(0)
+	, m_currentPageNumber(1)
 
-	m_sectionAttributesChanged(false),
-	m_numColumns(1),
-	m_textColumns(),
-	m_isTextColumnWithoutParagraph(false),
+	, m_sectionAttributesChanged(false)
+	, m_numColumns(1)
+	, m_textColumns()
+	, m_isTextColumnWithoutParagraph(false)
 
-	m_pageFormLength(11.0),
-	m_pageFormWidth(8.5f),
-	m_pageFormOrientationIsPortrait(true),
+	, m_pageFormLength(11.0)
+	, m_pageFormWidth(8.5)
+	, m_pageFormOrientationIsPortrait(true)
 
-	m_pageMarginLeft(1.0),
-	m_pageMarginRight(1.0),
-	m_pageMarginTop(1.0),
-	m_pageMarginBottom(1.0),
+	, m_pageMarginLeft(1.0)
+	, m_pageMarginRight(1.0)
+	, m_pageMarginTop(1.0)
+	, m_pageMarginBottom(1.0)
 
-	m_listOrderedLevels(),
+	, m_listOrderedLevels()
 
-	m_inSubDocument(false),
-	m_isNote(false),
-	m_subDocumentType(libwps::DOC_NONE)
+	, m_inSubDocument(false)
+	, m_isNote(false)
+	, m_subDocumentType(libwps::DOC_NONE)
 {
 	m_font.m_size=12.0;
 	m_font.m_name="Times New Roman";
@@ -192,9 +208,11 @@ WPSContentParsingState::~WPSContentParsingState()
 {
 }
 
-WPSContentListener::WPSContentListener(std::vector<WPSPageSpan> const &pageList, librevenge::RVNGTextInterface *documentInterface) :
-	m_ds(new WPSDocumentParsingState(pageList)), m_ps(new WPSContentParsingState), m_psStack(),
-	m_documentInterface(documentInterface)
+WPSContentListener::WPSContentListener(std::vector<WPSPageSpan> const &pageList, librevenge::RVNGTextInterface *documentInterface)
+	: m_ds(new WPSDocumentParsingState(pageList))
+	, m_ps(new WPSContentParsingState)
+	, m_psStack()
+	, m_documentInterface(documentInterface)
 {
 	_updatePageSpanDependent(true);
 }
@@ -370,16 +388,16 @@ void WPSContentListener::setParagraph(const WPSParagraph &para)
 	// check if we need to update the list
 	if (para.m_listLevelIndex >= 1)
 	{
-		WPSList::Level level = para.m_listLevel;
+		auto level = para.m_listLevel;
 		level.m_labelWidth = (para.m_margins[1]-level.m_labelIndent);
 		if (level.m_labelWidth<0.1)
 			level.m_labelWidth = 0.1;
 		level.m_labelIndent = 0;
 
-		shared_ptr<WPSList> theList = getCurrentList();
+		auto theList = getCurrentList();
 		if (!theList)
 		{
-			theList = shared_ptr<WPSList>(new WPSList);
+			theList = std::make_shared<WPSList>();
 			theList->set(para.m_listLevelIndex, level);
 			setCurrentList(theList);
 		}
@@ -392,13 +410,13 @@ void WPSContentListener::setParagraph(const WPSParagraph &para)
 ///////////////////
 // List: Minimal implementation
 ///////////////////
-void WPSContentListener::setCurrentList(shared_ptr<WPSList> list)
+void WPSContentListener::setCurrentList(std::shared_ptr<WPSList> list)
 {
 	m_ps->m_list=list;
 	if (list && list->getId() <= 0 && list->numLevels())
 		list->setId(++m_ds->m_newListId);
 }
-shared_ptr<WPSList> WPSContentListener::getCurrentList() const
+std::shared_ptr<WPSList> WPSContentListener::getCurrentList() const
 {
 	return m_ps->m_list;
 }
@@ -416,7 +434,7 @@ void WPSContentListener::insertField(WPSField const &field)
 		m_documentInterface->insertField(propList);
 		return;
 	}
-	librevenge::RVNGString text=field.getString();
+	auto text=field.getString();
 	if (!text.empty())
 		WPSContentListener::insertUnicodeString(text);
 	else
@@ -478,7 +496,7 @@ bool WPSContentListener::openSection(std::vector<int> colsWidth, librevenge::RVN
 		for (size_t col = 0; col < numCols; col++)
 		{
 			WPSColumnDefinition column;
-			column.m_width = factor*float(colsWidth[col]);
+			column.m_width = double(factor)*double(colsWidth[col]);
 			m_ps->m_textColumns[col] = column;
 		}
 	}
@@ -584,10 +602,10 @@ void WPSContentListener::_openPageSpan()
 		throw libwps::ParseException();
 	}
 	unsigned actPage = 0;
-	std::vector<WPSPageSpan>::iterator it = m_ds->m_pageList.begin();
+	auto it = m_ds->m_pageList.begin();
 	while (actPage < m_ps->m_currentPage)
 	{
-		actPage+=(unsigned) it++->getPageSpan();
+		actPage+=unsigned(it++->getPageSpan());
 		if (it == m_ds->m_pageList.end())
 		{
 			WPS_DEBUG_MSG(("WPSContentListener::_openPageSpan: can not find current page, revert to last page\n"));
@@ -665,9 +683,8 @@ void WPSContentListener::_openSection()
 		propList.insert("text:dont-balance-text-columns", false);
 
 	librevenge::RVNGPropertyListVector columns;
-	for (size_t i = 0; i < m_ps->m_textColumns.size(); i++)
+	for (auto const &col : m_ps->m_textColumns)
 	{
-		WPSColumnDefinition const &col = m_ps->m_textColumns[i];
 		librevenge::RVNGPropertyList column;
 		// The "style:rel-width" is expressed in twips (1440 twips per inch) and includes the left and right Gutter
 		column.insert("style:rel-width", col.m_width * 1440.0, librevenge::RVNG_TWIP);
@@ -780,7 +797,7 @@ void WPSContentListener::_appendParagraphProperties
 	if (!m_ps->m_inSubDocument && m_ps->m_firstParagraphInPageSpan)
 	{
 		unsigned actPage = 1;
-		std::vector<WPSPageSpan>::const_iterator it = m_ds->m_pageList.begin();
+		auto it = m_ds->m_pageList.begin();
 		while (actPage < m_ps->m_currentPage)
 		{
 			if (it == m_ds->m_pageList.end())
@@ -875,7 +892,7 @@ void WPSContentListener::_changeList()
 
 		if (m_ps->m_list->mustSendLevel(m_ps->m_paragraph.m_listLevelIndex))
 		{
-			if (actualListLevel == (size_t) m_ps->m_paragraph.m_listLevelIndex)
+			if (actualListLevel == size_t(m_ps->m_paragraph.m_listLevelIndex))
 			{
 				if (m_ps->m_listOrderedLevels[actualListLevel-1])
 					m_documentInterface->closeOrderedListLevel();
@@ -897,10 +914,10 @@ void WPSContentListener::_changeList()
 		m_ps->m_list->closeElement();
 	}
 
-	if ((int)actualListLevel == m_ps->m_paragraph.m_listLevelIndex) return;
+	if (int(actualListLevel) == m_ps->m_paragraph.m_listLevelIndex) return;
 
-	m_ps->m_listOrderedLevels.resize((size_t)m_ps->m_paragraph.m_listLevelIndex, false);
-	for (size_t i=actualListLevel+1; i<= (size_t)m_ps->m_paragraph.m_listLevelIndex; i++)
+	m_ps->m_listOrderedLevels.resize(size_t(m_ps->m_paragraph.m_listLevelIndex), false);
+	for (size_t i=actualListLevel+1; i<= size_t(m_ps->m_paragraph.m_listLevelIndex); i++)
 	{
 		librevenge::RVNGPropertyList propList;
 		m_ps->m_list->addLevelTo(int(i), propList);
@@ -1048,7 +1065,7 @@ void WPSContentListener::insertLabelNote(const NoteType noteType, librevenge::RV
 		m_ps->m_paragraph.m_listLevelIndex = 0;
 		_changeList(); // flush the list exterior
 		handleSubDocument(subDocument, libwps::DOC_NOTE);
-		m_ps->m_paragraph.m_listLevelIndex = (uint8_t)prevListLevel;
+		m_ps->m_paragraph.m_listLevelIndex = uint8_t(prevListLevel);
 	}
 	else
 	{
@@ -1182,8 +1199,8 @@ bool WPSContentListener::openGroup(WPSPosition const &pos)
 #if !defined(__clang__)
 	default:
 		WPS_DEBUG_MSG(("WPSContentListener::openGroup: UNKNOWN position, insert as char position\n"));
-#endif
 		WPS_FALLTHROUGH;
+#endif
 	case WPSPosition::CharBaseLine:
 	case WPSPosition::Char:
 		if (m_ps->m_isSpanOpened)
@@ -1191,6 +1208,8 @@ bool WPSContentListener::openGroup(WPSPosition const &pos)
 		else
 			_openSpan();
 		break;
+	case WPSPosition::Cell:
+		break;
 	}
 
 	librevenge::RVNGPropertyList propList;
@@ -1249,6 +1268,18 @@ bool WPSContentListener::_openFrame(WPSPosition const &pos, librevenge::RVNGProp
 		else
 			_openSpan();
 		break;
+	case WPSPosition::Cell:
+		if (!m_ps->m_isTableCellOpened)
+		{
+			WPS_DEBUG_MSG(("WPSTextListener::openFrame: called with Cell position not in a table cell\n"));
+			return false;
+		}
+		if (pos.m_anchorCellName.empty())
+		{
+			WPS_DEBUG_MSG(("WPSTextListener::openFrame: can not find the cell name\n"));
+			return false;
+		}
+		break;
 	default:
 		WPS_DEBUG_MSG(("WPSContentListener::openFrame: can not determine the anchor\n"));
 		return false;
@@ -1277,7 +1308,7 @@ void WPSContentListener::_handleFrameParameters
 (librevenge::RVNGPropertyList &propList, WPSPosition const &pos)
 {
 	Vec2f origin = pos.origin();
-	librevenge::RVNGUnit unit = pos.unit();
+	auto unit = pos.unit();
 	float inchFactor=pos.getInvUnitScale(librevenge::RVNG_INCH);
 	float pointFactor = pos.getInvUnitScale(librevenge::RVNG_POINT);
 
@@ -1285,8 +1316,8 @@ void WPSContentListener::_handleFrameParameters
 	propList.insert("svg:height", double(pos.size()[1]), unit);
 	if (pos.naturalSize().x() > 4*pointFactor && pos.naturalSize().y() > 4*pointFactor)
 	{
-		propList.insert("librevenge:naturalWidth", pos.naturalSize().x(), pos.unit());
-		propList.insert("librevenge:naturalHeight", pos.naturalSize().y(), pos.unit());
+		propList.insert("librevenge:naturalWidth", double(pos.naturalSize().x()), pos.unit());
+		propList.insert("librevenge:naturalHeight", double(pos.naturalSize().y()), pos.unit());
 	}
 
 	if (pos.m_wrapping ==  WPSPosition::WDynamic)
@@ -1306,14 +1337,14 @@ void WPSContentListener::_handleFrameParameters
 		                              "paragraph" : "paragraph-content");
 		propList.insert("style:vertical-rel", posRel);
 		propList.insert("style:horizontal-rel", posRel);
-		double w = m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft
-		           - m_ps->m_pageMarginRight - m_ps->m_paragraph.m_margins[1]
-		           - m_ps->m_paragraph.m_margins[2];
+		float w = float (m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft
+		                 - m_ps->m_pageMarginRight - m_ps->m_paragraph.m_margins[1]
+		                 - m_ps->m_paragraph.m_margins[2]);
 		w *= inchFactor;
 		switch (pos.m_xPos)
 		{
 		case WPSPosition::XRight:
-			if (origin[0] < 0.0 || origin[0] > 0.0)
+			if (origin[0] < 0.0f || origin[0] > 0.0f)
 			{
 				propList.insert("style:horizontal-pos", "from-left");
 				propList.insert("svg:x", double(origin[0] - pos.size()[0] + w), unit);
@@ -1322,10 +1353,10 @@ void WPSContentListener::_handleFrameParameters
 				propList.insert("style:horizontal-pos", "right");
 			break;
 		case WPSPosition::XCenter:
-			if (origin[0] < 0.0 || origin[0] > 0.0)
+			if (origin[0] < 0.0f || origin[0] > 0.0f)
 			{
 				propList.insert("style:horizontal-pos", "from-left");
-				propList.insert("svg:x", double(origin[0] - pos.size()[0]/2.0 + w/2.0), unit);
+				propList.insert("svg:x", double(origin[0] - pos.size()[0]/2.0f + w/2.0f), unit);
 			}
 			else
 				propList.insert("style:horizontal-pos", "center");
@@ -1333,7 +1364,7 @@ void WPSContentListener::_handleFrameParameters
 		case WPSPosition::XLeft:
 		case WPSPosition::XFull:
 		default:
-			if (origin[0] < 0.0 || origin[0] > 0.0)
+			if (origin[0] < 0.0f || origin[0] > 0.0f)
 			{
 				propList.insert("style:horizontal-pos", "from-left");
 				propList.insert("svg:x", double(origin[0]), unit);
@@ -1343,7 +1374,7 @@ void WPSContentListener::_handleFrameParameters
 			break;
 		}
 
-		if (origin[1] < 0.0 || origin[1] > 0.0)
+		if (origin[1] < 0.0f || origin[1] > 0.0f)
 		{
 			propList.insert("style:vertical-pos", "from-top");
 			propList.insert("svg:y", double(origin[1]), unit);
@@ -1358,8 +1389,8 @@ void WPSContentListener::_handleFrameParameters
 		// Page position seems to do not use the page margin...
 		propList.insert("text:anchor-type", "page");
 		if (pos.page() > 0) propList.insert("text:anchor-page-number", pos.page());
-		double  w = m_ps->m_pageFormWidth;
-		double h = m_ps->m_pageFormLength;
+		float w = float(m_ps->m_pageFormWidth);
+		float h = float(m_ps->m_pageFormLength);
 		w *= inchFactor;
 		h *= inchFactor;
 
@@ -1367,14 +1398,14 @@ void WPSContentListener::_handleFrameParameters
 		propList.insert("style:vertical-rel", posRel);
 		propList.insert("style:horizontal-rel", posRel);
 
-		double newPosition;
+		float newPosition;
 		switch (pos.m_yPos)
 		{
 		case WPSPosition::YFull:
 			propList.insert("svg:height", double(h), unit);
 			WPS_FALLTHROUGH;
 		case WPSPosition::YTop:
-			if (origin[1] < 0.0 || origin[1] > 0.0)
+			if (origin[1] < 0.0f || origin[1] > 0.0f)
 			{
 				propList.insert("style:vertical-pos", "from-top");
 				newPosition = origin[1];
@@ -1386,10 +1417,10 @@ void WPSContentListener::_handleFrameParameters
 				propList.insert("style:vertical-pos", "top");
 			break;
 		case WPSPosition::YCenter:
-			if (origin[1] < 0.0 || origin[1] > 0.0)
+			if (origin[1] < 0.0f || origin[1] > 0.0f)
 			{
 				propList.insert("style:vertical-pos", "from-top");
-				newPosition = (h - pos.size()[1])/2.0;
+				newPosition = (h - pos.size()[1])/2.0f;
 				if (newPosition > h -pos.size()[1]) newPosition = h - pos.size()[1];
 				propList.insert("svg:y", double(newPosition), unit);
 			}
@@ -1397,7 +1428,7 @@ void WPSContentListener::_handleFrameParameters
 				propList.insert("style:vertical-pos", "middle");
 			break;
 		case WPSPosition::YBottom:
-			if (origin[1] < 0.0 || origin[1] > 0.0)
+			if (origin[1] < 0.0f || origin[1] > 0.0f)
 			{
 				propList.insert("style:vertical-pos", "from-top");
 				newPosition = h - pos.size()[1]-origin[1];
@@ -1418,7 +1449,7 @@ void WPSContentListener::_handleFrameParameters
 			propList.insert("svg:width", double(w), unit);
 			WPS_FALLTHROUGH;
 		case WPSPosition::XLeft:
-			if (origin[0] < 0.0 || origin[0] > 0.0)
+			if (origin[0] < 0.0f || origin[0] > 0.0f)
 			{
 				propList.insert("style:horizontal-pos", "from-left");
 				propList.insert("svg:x", double(origin[0]), unit);
@@ -1427,7 +1458,7 @@ void WPSContentListener::_handleFrameParameters
 				propList.insert("style:horizontal-pos", "left");
 			break;
 		case WPSPosition::XRight:
-			if (origin[0] < 0.0 || origin[0] > 0.0)
+			if (origin[0] < 0.0f || origin[0] > 0.0f)
 			{
 				propList.insert("style:horizontal-pos", "from-left");
 				propList.insert("svg:x",double(w - pos.size()[0] + origin[0]), unit);
@@ -1437,10 +1468,10 @@ void WPSContentListener::_handleFrameParameters
 			break;
 		case WPSPosition::XCenter:
 		default:
-			if (origin[0] < 0.0 || origin[0] > 0.0)
+			if (origin[0] < 0.0f || origin[0] > 0.0f)
 			{
 				propList.insert("style:horizontal-pos", "from-left");
-				propList.insert("svg:x", double((w - pos.size()[0])/2. + origin[0]), unit);
+				propList.insert("svg:x", double((w - pos.size()[0])/2 + origin[0]), unit);
 			}
 			else
 				propList.insert("style:horizontal-pos", "center");
@@ -1448,6 +1479,17 @@ void WPSContentListener::_handleFrameParameters
 		}
 		return;
 	}
+	if (pos.m_anchorTo == WPSPosition::Cell)
+	{
+		if (!pos.m_anchorCellName.empty())
+			propList.insert("table:end-cell-address", pos.m_anchorCellName);
+		// todo: implement also different m_xPos and m_yPos
+		if (origin[0] < 0 || origin[0] > 0)
+			propList.insert("svg:x", double(origin[0]), unit);
+		if (origin[1] < 0 || origin[1] > 0)
+			propList.insert("svg:y", double(origin[1]), unit);
+		return;
+	}
 	if (pos.m_anchorTo != WPSPosition::Char &&
 	        pos.m_anchorTo != WPSPosition::CharBaseLine) return;
 
@@ -1460,7 +1502,7 @@ void WPSContentListener::_handleFrameParameters
 	{
 	case WPSPosition::YFull:
 	case WPSPosition::YTop:
-		if (origin[1] < 0.0 || origin[1] > 0.0)
+		if (origin[1] < 0.0f || origin[1] > 0.0f)
 		{
 			propList.insert("style:vertical-pos", "from-top");
 			propList.insert("svg:y", double(origin[1]), unit);
@@ -1469,17 +1511,17 @@ void WPSContentListener::_handleFrameParameters
 			propList.insert("style:vertical-pos", "top");
 		break;
 	case WPSPosition::YCenter:
-		if (origin[1] < 0.0 || origin[1] > 0.0)
+		if (origin[1] < 0.0f || origin[1] > 0.0f)
 		{
 			propList.insert("style:vertical-pos", "from-top");
-			propList.insert("svg:y", double(origin[1] - pos.size()[1]/2.0), unit);
+			propList.insert("svg:y", double(origin[1] - pos.size()[1]/2.0f), unit);
 		}
 		else
 			propList.insert("style:vertical-pos", "middle");
 		break;
 	case WPSPosition::YBottom:
 	default:
-		if (origin[1] < 0.0 || origin[1] > 0.0)
+		if (origin[1] < 0.0f || origin[1] > 0.0f)
 		{
 			propList.insert("style:vertical-pos", "from-top");
 			propList.insert("svg:y", double(origin[1] - pos.size()[1]), unit);
@@ -1517,17 +1559,18 @@ void WPSContentListener::handleSubDocument(WPSSubDocumentPtr &subDocument, libwp
 	case libwps::DOC_NOTE:
 	case libwps::DOC_TABLE:
 	case libwps::DOC_COMMENT_ANNOTATION:
+	case libwps::DOC_CHART_ZONE:
 	default:
 		break;
 	}
 
 	// Check whether the document is calling itself
 	bool sendDoc = true;
-	for (size_t i = 0; i < m_ds->m_subDocuments.size(); i++)
+	for (const auto &i : m_ds->m_subDocuments)
 	{
 		if (!subDocument)
 			break;
-		if (subDocument == m_ds->m_subDocuments[i])
+		if (*subDocument == i)
 		{
 			WPS_DEBUG_MSG(("WPSContentListener::handleSubDocument: recursif call, stop...\n"));
 			sendDoc = false;
@@ -1539,10 +1582,10 @@ void WPSContentListener::handleSubDocument(WPSSubDocumentPtr &subDocument, libwp
 		if (subDocument)
 		{
 			m_ds->m_subDocuments.push_back(subDocument);
-			shared_ptr<WPSContentListener> listen(this, WPS_shared_ptr_noop_deleter<WPSContentListener>());
+			std::shared_ptr<WPSContentListener> listen(this, WPS_shared_ptr_noop_deleter<WPSContentListener>());
 			try
 			{
-				WPSTextSubDocument *docu=dynamic_cast<WPSTextSubDocument *>(subDocument.get());
+				auto *docu=dynamic_cast<WPSTextSubDocument *>(subDocument.get());
 				if (docu)
 					docu->parse(listen, subDocumentType);
 				else
@@ -1572,6 +1615,7 @@ void WPSContentListener::handleSubDocument(WPSSubDocumentPtr &subDocument, libwp
 	case libwps::DOC_NOTE:
 	case libwps::DOC_TABLE:
 	case libwps::DOC_COMMENT_ANNOTATION:
+	case libwps::DOC_CHART_ZONE:
 	default:
 		break;
 	}
@@ -1626,16 +1670,15 @@ void WPSContentListener::openTable(std::vector<float> const &colWidth, libreveng
 	float tableWidth = 0;
 	librevenge::RVNGPropertyListVector columns;
 
-	size_t nCols = colWidth.size();
-	for (size_t c = 0; c < nCols; c++)
+	for (auto const &w : colWidth)
 	{
 		librevenge::RVNGPropertyList column;
-		column.insert("style:column-width", colWidth[c], unit);
+		column.insert("style:column-width", double(w), unit);
 		columns.append(column);
 
-		tableWidth += colWidth[c];
+		tableWidth += w;
 	}
-	propList.insert("style:width", tableWidth, unit);
+	propList.insert("style:width", double(tableWidth), unit);
 	propList.insert("librevenge:table-columns", columns);
 	m_documentInterface->openTable(propList);
 	m_ps->m_isTableOpened = true;
@@ -1672,9 +1715,9 @@ void WPSContentListener::openTableRow(float h, librevenge::RVNGUnit unit, bool h
 	propList.insert("librevenge:is-header-row", headerRow);
 
 	if (h > 0)
-		propList.insert("style:row-height", h, unit);
+		propList.insert("style:row-height", double(h), unit);
 	else if (h < 0)
-		propList.insert("style:min-row-height", -h, unit);
+		propList.insert("style:min-row-height", -double(h), unit);
 	m_documentInterface->openTableRow(propList);
 	m_ps->m_isTableRowOpened = true;
 }
@@ -1752,9 +1795,9 @@ void WPSContentListener::closeTableCell()
 ///////////////////
 
 // ---------- state stack ------------------
-shared_ptr<WPSContentParsingState> WPSContentListener::_pushParsingState()
+std::shared_ptr<WPSContentParsingState> WPSContentListener::_pushParsingState()
 {
-	shared_ptr<WPSContentParsingState> actual = m_ps;
+	auto actual = m_ps;
 	m_psStack.push_back(actual);
 	m_ps.reset(new WPSContentParsingState);
 
